<?php

namespace Bydedi\Tracking\Google;

use Symfony\Component\HttpFoundation\Request;
use Client;
use Panier;

/**
 * Class TagManager
 * @package Bydedi\Tracking\Google
 */
class TagManager
{
    /**
     * @var bool
     */
    protected static $isInit = false;

    /**
     * @var array|null $config
     */
    protected static $config = null;

    /**
     * @var array|null $events
     */
    protected static $events = null;

    /**
     * @param array $config
     */
    public static function init($config = array())
    {
        self::$config = $config;
    }

    /**
     * Check if it's initialized.
     * If not trigger a warning error
     */
    protected static function triggerErrorIfNotInit()
    {
        if (is_null(self::$config)) {
            trigger_error("Google Tag Manager n'est pas initialisé", E_USER_WARNING);
        }
    }

    protected static function assign(Request $request, $data_layer, $param)
    {
        $infos = static::common($request, $param);

        $tracker = "";

        if (!empty($infos)) {
            $data_layer = array_merge($infos[0], $data_layer);
            $tracker.= $infos[1];
        }

        $data_layer = '<script>var dataLayer = [];dataLayer.push(' . json_encode($data_layer) . ');</script>';

        if (isset($param['smarty'])) {
            $param['smarty']->reassign(self::$config['position_datalayer'], $data_layer);
            $param['smarty']->reassign(self::$config['position'], $infos[2] . $tracker);
        }
    }

    public static function common(Request $request, $param)
    {
        if (self::$isInit == true) {
            return false;
        }

        self::$isInit = true;

        $data_layer = array(
            "env_template" => $request->get('_route'),
        );

        $data_layer = array_merge($data_layer, static::user($request, $param, true));

        $tracker = "
            <!-- Google Tag Manager -->
            <noscript><iframe src=\"//www.googletagmanager.com/ns.html?id=%s\"
            height=\"0\" width=\"0\" style=\"display:none;visibility:hidden\"></iframe></noscript>
            <script>(function(w,d,s,l,i){w[l]=w[l]||[];w[l].push({'gtm.start':
            new Date().getTime(),event:'gtm.js'});var f=d.getElementsByTagName(s)[0],
            j=d.createElement(s),dl=l!='dataLayer'?'&l='+l:'';j.async=true;j.src=
            '//www.googletagmanager.com/gtm.js?id='+i+dl;f.parentNode.insertBefore(j,f);
            })(window,document,'script','dataLayer','%s');</script>
            <!-- End Google Tag Manager -->
        ";

        $tracker = sprintf($tracker, self::$config['id'], self::$config['id']);

        $events = (is_null(self::$events) == false) ? self::getPushedEventsJs() : '';

        return array($data_layer, $tracker, $events);
    }

    public static function getVisites(Request $request, $param = array())
    {
        if (self::$isInit == true) {
            return false;
        }

        self::assign($request, array(), $param);
    }

    public static function home(Request $request, $param = array())
    {
        $data_layer = array(

        );

        self::assign($request, $data_layer, $param);
    }

    public static function category(Request $request, $param = array())
    {
        $data_layer = array(
            'breadcrumb_id' => $param['breadcrumb_id'],
            'breadcrumb_label' => $param['breadcrumb_label'],
            'page_id' => $param['page_id'],
        );

        static::assign($request, $data_layer, $param);
    }

    public static function search(Request $request, $param = array())
    {
        $data_layer = array(
            'search_keywords' => $request->get('motclef'),
            'search_results_number' => '',
        );

        static::assign($request, $data_layer, $param);
    }

    public static function product(Request $request, $param = array(), $onlyDatalayer = false)
    {
        $data_layer = array(
            'product_sku' => $param['produit_ref'],
            'product_name' => $param['produit_nom_nom'],
            'product_unitprice_ati' => $param['produit_prix_ttc'],
        );

        if (!empty($param['vars'])) {
            $data_layer = array_merge($data_layer, $param['vars']);
        }

        if ($onlyDatalayer) {
            return $data_layer;
        }

        static::assign($request, $data_layer, $param);
    }

    public static function user(Request $request, $param = array(), $onlyDatalayer = false)
    {
        $session = $request->getSession();

        $data_layer = array(
            'user_id' => '',
            'user_status' => 'unknown',
        );

        if ($session->has('client_id')) {

            $client = new Client($session->get('client_id'));

            $data_layer['user_id'] = $client->client_id;
            $data_layer['user_status'] = 'identified';
        }

        if ($onlyDatalayer) {
            return $data_layer;
        }

        static::assign($request, $data_layer, $param);
    }

    public static function cart(Request $request, $param = array())
    {
        $panier = Panier::getInstance();
        list($aPayerHT, $aPayerTTC) = $panier->getAPayer();

        $data_layer = array(
            'basket_amount_ati' => round($aPayerTTC, 2),
            'basket_product_number' => $panier->getCountProducts(),
        );

        if (!empty($param['vars'])) {
            $data_layer = array_merge($data_layer, $param['vars']);
        }

        static::assign($request, $data_layer, $param);
    }

    public static function paiement(Request $request, $param = array())
    {
        $data_layer = array(
            'order_id' => $param['extra']['commande_id'],
            'order_amount_ati_with_sf' => $param['extra']['commande_total_ttc'],
            'order_payment_methods' => $param['extra']['paiement_nom_nom'],
            'order_products_number' => 0,
            'order_products' => [],
        );

        foreach ($param['extra']['produits'] as $produit) {
            $item = [
                'order_product_id' => $produit['commande_produit_id'],
                'order_product_name' => $produit['produit_nom'],
                'order_product_sku' => $produit['produit_ref'],
                'order_product_quantity' => $produit['qte'],
                'order_product_unitprice_ati' => round($produit['prix_ttc'], 2),
                'order_product_breadcrumb_id' => '',
                'order_product_breadcrumb_label' => '',
                'order_product_designer' => $produit['marque_nom'],
            ];

            $data_layer['order_products'][] = $item;
            $data_layer['order_products_number']+= $produit['qte'];

        }

        static::assign($request, $data_layer, $param);
    }

    public static function pushEvent($node, $event, $category, $action)
    {
        self::$events[] = [
            'node' => $node,
            'event' => $event,
            'category' => $category,
            'action' => $action,
        ];
    }

    public static function getPushedEvents()
    {
        return self::$events;
    }

    public static function getPushedEventsJs()
    {
        $scripts = [];
        $mask = "$(document).on('click', '%s', function(){
                    dataLayer.push({
                        'event': '%s',
                        'eventcategory': '%s',
                        'eventaction': '%s',
                        'eventlabel': 'url: ' + window.location.href,
                        });
                    });";

        foreach (self::$events as $event) {
            $scripts[] = sprintf($mask, $event['node'], $event['event'], $event['category'], $event['action']);
        }

        return sprintf('<script>$(document).ready(function(){%s});</script>', implode('', $scripts));
    }
}
